from gmshparser.helpers import parse_ints, parse_floats
import io


def test_parse_ints():
    s = io.StringIO("1 2 3 4")
    assert parse_ints(s) == [1, 2, 3, 4]


def test_parse_floats():
    s = io.StringIO("1.1 2.2 3.3 4.4")
    assert parse_floats(s) == [1.1, 2.2, 3.3, 4.4]



from typing import List, TextIO


def parse_ints(io: TextIO) -> List[int]:
    """Parse first line of io to list of integers.

    Parameters
    ----------
    io :: TextIO
        Object supporting `readline()`

    Returns
    -------
    integers :: List[int]
        A list of integers

    Examples
    --------
    >>> data = StringIO("1 2 3 4")
    >>> parse_ints(data)
    [1, 2, 3, 4]
    """
    line = io.readline()
    line = line.strip()
    parts = line.split(" ")
    ints = map(int, parts)
    return list(ints)


def parse_floats(io: TextIO) -> List[float]:
    """Parse first line of io to list of floats.

    Parameters
    ----------
    io :: TextIO
        Object supporting `readline()`

    Returns
    -------
    floats :: List[float]
        A list of floats

    Examples
    --------
    >>> data = StringIO("1.1 2.2 3.3 4.4")
    >>> parse_floats(data)
    [1.1, 2.2, 3.3, 4.4]
    """
    line = io.readline()
    line = line.strip()
    parts = line.split(" ")
    ints = map(float, parts)
    return list(ints)



from gmshparser.element import Element


def test_element():
    element = Element()
    element.set_tag(1)
    element.set_connectivity([1, 2, 3, 4])
    assert element.get_tag() == 1
    assert element.get_connectivity() == [1, 2, 3, 4]



from typing import List
from gmshparser.node_entity import NodeEntity
from gmshparser.element_entity import ElementEntity
from io import StringIO


class Mesh(object):

    """Mesh is the main class of the package."""

    def __init__(self):
        self.name_ = "New Mesh"
        self.version_ = "4.1"
        self.ascii_ = True
        self.precision_ = 8  # t_size
        self.number_of_node_entities_ = 0
        self.number_of_nodes_ = 0
        self.min_node_tag_ = 0
        self.max_node_tag_ = 0
        self.node_entities_ = {}
        self.number_of_element_entities_ = 0
        self.number_of_elements_ = 0
        self.min_element_tag_ = 0
        self.max_element_tag_ = 0
        self.element_entities_ = {}

    def set_name(self, name: str):
        """Set the name of the mesh."""
        self.name_ = name

    def get_name(self) -> str:
        """Get the name of the mesh."""
        return self.name_

    def set_version(self, version: str):
        """Set the version of the Mesh object"""
        self.version_ = version

    def get_version(self) -> str:
        """Get the version of the Mesh object"""
        return self.version_

    def set_ascii(self, is_ascii: bool):
        """Set a boolean flag whether this mesh is ASCII or binary"""
        self.ascii_ = is_ascii

    def get_ascii(self) -> bool:
        """Get a boolean flag whether this mesh is ASCII of binary"""
        return self.ascii_

    def set_precision(self, precision: int):
        """Set the precision of the mesh (8)"""
        self.precision_ = precision

    def get_precision(self) -> int:
        """Get the precision of the mesh"""
        return self.precision_

    def set_number_of_node_entities(self, number_of_node_entities: int):
        """Set number of node entities."""
        self.number_of_node_entities_ = number_of_node_entities

    def get_number_of_node_entities(self) -> int:
        """Get number of node entities."""
        return self.number_of_node_entities_

    def set_number_of_nodes(self, number_of_nodes: int):
        """Set number of nodes."""
        self.number_of_nodes_ = number_of_nodes

    def get_number_of_nodes(self) -> int:
        """Get number of nodes."""
        return self.number_of_nodes_

    def set_min_node_tag(self, min_node_tag: int):
        """Set node minimum tag."""
        self.min_node_tag_ = min_node_tag

    def get_min_node_tag(self) -> int:
        """Get node minimum tag."""
        return self.min_node_tag_

    def set_max_node_tag(self, max_node_tag: int):
        """Set node maximum tag."""
        self.max_node_tag_ = max_node_tag

    def get_max_node_tag(self) -> int:
        """Get node maximum tag."""
        return self.max_node_tag_

    def has_node_entity(self, node_entity_tag) -> bool:
        """Test does mesh have entity with `node_entity_tag`."""
        return node_entity_tag in self.node_entities_

    def add_node_entity(self, node_entity: NodeEntity):
        """Add node entity to mesh."""
        self.node_entities_[node_entity.get_tag()] = node_entity

    def get_node_entity(self, tag: int):
        """Get node entity based on tag."""
        return self.node_entities_[tag]

    def get_node_entities(self) -> List[NodeEntity]:
        """Get all node entities of mesh."""
        return self.node_entities_.values()

    def set_number_of_element_entities(self, number_of_element_entities: int):
        """Set number of element entities."""
        self.number_of_element_entities_ = number_of_element_entities

    def get_number_of_element_entities(self) -> int:
        """Get number of element entities."""
        return self.number_of_element_entities_

    def set_number_of_elements(self, number_of_elements: int):
        """Set number of elements."""
        self.number_of_elements_ = number_of_elements

    def get_number_of_elements(self) -> int:
        """Get number of elements."""
        return self.number_of_elements_

    def set_min_element_tag(self, min_element_tag: int):
        """Set element minimum tag."""
        self.min_element_tag_ = min_element_tag

    def get_min_element_tag(self) -> int:
        """Get element minimum tag."""
        return self.min_element_tag_

    def set_max_element_tag(self, max_element_tag: int):
        """Set element maximum tag."""
        self.max_element_tag_ = max_element_tag

    def get_max_element_tag(self) -> int:
        """Get element maximum tag."""
        return self.max_element_tag_

    def has_element_entity(self, element_entity_tag) -> bool:
        """Test does mesh have element entity with `element_entity_tag`."""
        return element_entity_tag in self.element_entities_

    def add_element_entity(self, element_entity: ElementEntity):
        """Add element entity to mesh."""
        self.element_entities_[element_entity.get_tag()] = element_entity

    def get_element_entity(self, tag: int) -> ElementEntity:
        """Get element entity based on tag."""
        return self.element_entities_[tag]

    def get_element_entities(self) -> List[ElementEntity]:
        """Get all element entities as dictionary."""
        return self.element_entities_.values()

    def __str__(self):
        io = StringIO()
        io.write("Mesh name: %s\n" % self.get_name())
        io.write("Mesh version: %s\n" % self.get_version())
        io.write("Number of nodes: %s\n" % self.get_number_of_nodes())
        io.write("Minimum node tag: %s\n" % self.get_min_node_tag())
        io.write("Maximum node tag: %s\n" % self.get_max_node_tag())
        io.write("Number of node entities: %s\n" %
                 self.get_number_of_node_entities())
        io.write("Number of elements: %s\n" % self.get_number_of_elements())
        io.write("Minimum element tag: %s\n" % self.get_min_element_tag())
        io.write("Maximum element tag: %s\n" % self.get_max_element_tag())
        io.write("Number of element entities: %s\n" %
                 self.get_number_of_element_entities())
        return io.getvalue()



from typing import Tuple


class Node(object):

    """ Node. """

    def __init__(self):
        self.tag_ = -1
        self.coordinates_ = (None, None, None)

    def set_tag(self, tag: int):
        """Set node tag (node id)."""
        self.tag_ = tag

    def get_tag(self) -> int:
        """Get node tag (node id)."""
        return self.tag_

    def set_coordinates(self, coordinates: Tuple[float, float, float]):
        """Set the coordinates of the node."""
        self.coordinates_ = coordinates

    def get_coordinates(self) -> Tuple[float, float, float]:
        """Get the coordinates of the node."""
        return self.coordinates_



from gmshparser.mesh_format_parser import MeshFormatParser
from gmshparser.mesh import Mesh
import io


def test_mesh_format_parser():
    parser = MeshFormatParser()
    mesh = Mesh()
    data = io.StringIO("4.1 0 8")
    parser.parse(mesh, data)
    assert mesh.get_version() == 4.1
    assert mesh.get_ascii() is True
    assert mesh.get_precision() == 8



from gmshparser.node import Node


def test_node():
    node = Node()
    node.set_tag(1)
    node.set_coordinates((1.0, 2.0, 3.0))
    assert node.get_tag() == 1
    assert node.get_coordinates() == (1.0, 2.0, 3.0)



from typing import List
from io import StringIO
from .element import Element


class ElementEntity(object):

    """ ElementEntity class holds elements for one block. """

    def __init__(self):
        self.dimension_ = -1
        self.tag_ = -1
        self.element_type_ = -1
        self.number_of_elements_ = -1
        self.elements_ = {}

    def set_dimension(self, dimension: int):
        """Set the dimension of element entity."""
        self.dimension_ = dimension

    def get_dimension(self) -> int:
        """Get the dimension of the element entity."""
        return self.dimension_

    def set_tag(self, tag: int):
        """Set the tag of the element entity."""
        self.tag_ = tag

    def get_tag(self) -> int:
        """Get the tag of the element entity."""
        return self.tag_

    def set_element_type(self, element_type: int):
        """Set element type in element entity."""
        self.element_type_ = element_type

    def get_element_type(self) -> int:
        """Get element type in element entity."""
        return self.element_type_

    def set_number_of_elements(self, number_of_elements: int):
        """Set the number of elements in entity."""
        self.number_of_elements_ = number_of_elements

    def get_number_of_elements(self) -> int:
        """Get the number of elements in entity."""
        return self.number_of_elements_

    def add_element(self, element: Element):
        """Add a new element to the entity."""
        self.elements_[element.get_tag()] = element

    def get_element(self, tag: int) -> Element:
        """Get an element from the entity."""
        return self.elements_[tag]

    def get_elements(self) -> List[Element]:
        """Return all the elements of this entity."""
        return self.elements_.values()

    def __str__(self):
        io = StringIO()
        io.write("---- Element entity # %s ----\n" % self.get_tag())
        io.write("Dimension: %s\n" % self.get_dimension())
        io.write("Element type: %s\n" % self.get_element_type())
        io.write("Number of elements: %s\n" % self.get_number_of_elements())
        return io.getvalue()



from gmshparser.nodes_parser import NodesParser
from gmshparser.mesh import Mesh
from io import StringIO

__content__ = """
$Nodes
1 6 1 6
2 1 0 6
1
2
3
4
5
6
0. 0. 0.
1. 0. 0.
1. 1. 0.
0. 1. 0.
2. 0. 0.
2. 1. 0.
$EndNodes
"""


def test_nodes_parser():
    parser = NodesParser()
    mesh = Mesh()
    data = StringIO(__content__.strip())
    parser.parse(mesh, data)

    assert mesh.get_number_of_nodes() == 6
    assert mesh.get_min_node_tag() == 1
    assert mesh.get_max_node_tag() == 6
    assert mesh.has_node_entity(1)

    entity = mesh.get_node_entity(1)
    assert entity.get_tag() == 1
    assert entity.get_dimension() == 2
    assert entity.get_number_of_parametric_coordinates() == 0
    assert entity.get_number_of_nodes() == 6

    assert entity.get_node(1).get_tag() == 1
    assert entity.get_node(1).get_coordinates() == (0.0, 0.0, 0.0)
    assert entity.get_node(2).get_tag() == 2
    assert entity.get_node(2).get_coordinates() == (1.0, 0.0, 0.0)
    assert entity.get_node(3).get_tag() == 3
    assert entity.get_node(3).get_coordinates() == (1.0, 1.0, 0.0)
    assert entity.get_node(4).get_tag() == 4
    assert entity.get_node(4).get_coordinates() == (0.0, 1.0, 0.0)
    assert entity.get_node(5).get_tag() == 5
    assert entity.get_node(5).get_coordinates() == (2.0, 0.0, 0.0)
    assert entity.get_node(6).get_tag() == 6
    assert entity.get_node(6).get_coordinates() == (2.0, 1.0, 0.0)



from typing import TextIO
from .mesh import Mesh
from .mesh_format_parser import MeshFormatParser
from .nodes_parser import NodesParser
from .elements_parser import ElementsParser
from .abstract_parser import AbstractParser


DEFAULT_PARSERS = [
    MeshFormatParser,
    NodesParser,
    ElementsParser,
]


class MainParser(AbstractParser):

    """ The main parser class, using other parsers. """

    def __init__(self, parsers=DEFAULT_PARSERS):
        self.parsers = parsers

    def parse(self, mesh: Mesh, io: TextIO) -> None:
        for line in io:
            line = line.strip()
            for parser in self.parsers:
                if parser.get_section_name() == line:
                    try:
                        parser.parse(mesh, io)
                    except Exception:
                        print("Unable to parse section %s from mesh!" % line)
                        raise



from gmshparser.element_entity import ElementEntity


class ElementMock(object):
    def __init__(self):
        pass

    def get_tag(self):
        return 42


__expected_str__ = """
---- Element entity # 1 ----
Dimension: 2
Element type: 3
Number of elements: 1
"""


def test_element_entity():
    ee = ElementEntity()
    ee.set_dimension(2)
    ee.set_tag(1)
    ee.set_number_of_elements(1)
    ee.set_element_type(3)
    element = ElementMock()
    ee.add_element(element)
    assert ee.get_dimension() == 2
    assert ee.get_tag() == 1
    assert ee.get_number_of_elements() == 1
    assert ee.get_element_type() == 3
    assert ee.get_element(42) == element
    assert len(ee.get_elements()) == 1
    assert ee.__str__() == __expected_str__.lstrip()



from gmshparser.node_entity import NodeEntity


def test_node_entity():
    ne = NodeEntity()
    ne.set_dimension(2)
    ne.set_tag(1)
    ne.set_number_of_parametric_coordinates(0)
    ne.set_number_of_nodes(1)
    assert ne.get_dimension() == 2
    assert ne.get_tag() == 1
    assert ne.get_number_of_parametric_coordinates() == 0
    assert ne.get_number_of_nodes() == 1
    assert len(ne.get_nodes()) == 0



from gmshparser.elements_parser import ElementsParser
from gmshparser.mesh import Mesh
from io import StringIO

__content__ = """
$Elements
1 2 1 2
2 1 3 2
1 1 2 3 4
2 2 5 6 3
$EndElements
"""


def test_elements_parser():
    parser = ElementsParser()
    mesh = Mesh()
    data = StringIO(__content__.strip())
    parser.parse(mesh, data)

    assert mesh.get_number_of_elements() == 2
    assert mesh.get_min_element_tag() == 1
    assert mesh.get_max_element_tag() == 2
    assert mesh.has_element_entity(1)

    entity = mesh.get_element_entity(1)
    assert entity.get_tag() == 1
    assert entity.get_dimension() == 2
    assert entity.get_element_type() == 3
    assert entity.get_number_of_elements() == 2

    assert entity.get_element(1).get_tag() == 1
    assert entity.get_element(1).get_connectivity() == [1, 2, 3, 4]
    assert entity.get_element(2).get_tag() == 2
    assert entity.get_element(2).get_connectivity() == [2, 5, 6, 3]



import pytest
from gmshparser.abstract_parser import AbstractParser


def test_abstract_parser():
    with pytest.raises(NotImplementedError):
        AbstractParser.get_section_name()
    p = AbstractParser()
    with pytest.raises(NotImplementedError):
        p.parse(1, 2)



from typing import List


class Element(object):

    """ Element. """

    def __init__(self):
        self.tag_ = -1
        self.connectivity_ = []

    def set_tag(self, tag: int):
        """Set element tag."""
        self.tag_ = tag

    def get_tag(self):
        """Get element tag."""
        return self.tag_

    def set_connectivity(self, connectivity: List[int]):
        """Set element connectivity."""
        self.connectivity_ = connectivity

    def get_connectivity(self) -> List[int]:
        """Get element connectivity."""
        return self.connectivity_



from gmshparser import parse
import os

__content__ = """
$MeshFormat
4.1 0 8
$EndMeshFormat
"""


def test_parse(tmpdir):
    fh = tmpdir.join("mesh1.msh")
    fh.write(__content__.strip())
    filename = os.path.join(fh.dirname, fh.basename)
    mesh = parse(filename)
    assert mesh.get_name().endswith("mesh1.msh")
    assert mesh.get_version() == 4.1



from typing import TextIO
from .node_entity import NodeEntity
from .node import Node
from .abstract_parser import AbstractParser
from .mesh import Mesh
from .helpers import parse_ints, parse_floats


class NodesParser(AbstractParser):

    @staticmethod
    def get_section_name():
        return "$Nodes"

    @staticmethod
    def parse(mesh: Mesh, io: TextIO) -> None:
        line = io.readline()
        if line.startswith("$Nodes"):
            line = io.readline()
        meta = list(map(int, line.strip().split(" ")))

        mesh.set_number_of_node_entities(meta[0])
        mesh.set_number_of_nodes(meta[1])
        mesh.set_min_node_tag(meta[2])
        mesh.set_max_node_tag(meta[3])
        for i in range(mesh.get_number_of_node_entities()):
            emeta = parse_ints(io)
            entity = NodeEntity()
            entity.set_dimension(emeta[0])
            entity.set_tag(emeta[1])
            entity.set_number_of_parametric_coordinates(emeta[2])
            entity.set_number_of_nodes(emeta[3])
            node_tags = []
            for j in range(entity.get_number_of_nodes()):
                tag = int(io.readline())
                node = Node()
                node.set_tag(tag)
                node_tags.append(tag)
                entity.add_node(node)
            for tag in node_tags:
                coordinates = parse_floats(io)
                node = entity.get_node(tag)
                node.set_coordinates(tuple(coordinates))
            mesh.add_node_entity(entity)



from gmshparser.mesh import Mesh

str_expected = """
Mesh name: testmesh.msh
Mesh version: 4.1
Number of nodes: 3
Minimum node tag: 1
Maximum node tag: 3
Number of node entities: 1
Number of elements: 2
Minimum element tag: 1
Maximum element tag: 2
Number of element entities: 1
"""


def test_mesh():
    m = Mesh()
    m.set_name("testmesh.msh")
    m.set_version(4.1)
    m.set_ascii(True)
    m.set_precision(8)
    m.set_min_node_tag(1)
    m.set_max_node_tag(3)
    m.set_number_of_node_entities(1)
    m.set_number_of_nodes(3)
    m.set_min_element_tag(1)
    m.set_max_element_tag(2)
    m.set_number_of_element_entities(1)
    m.set_number_of_elements(2)
    assert m.get_name() == "testmesh.msh"
    assert m.get_version() == 4.1
    assert m.get_ascii() is True
    assert m.get_precision() == 8
    assert m.get_min_node_tag() == 1
    assert m.get_max_node_tag() == 3
    assert m.get_number_of_node_entities() == 1
    assert m.get_number_of_nodes() == 3
    assert m.__str__() == str_expected.lstrip()
    assert len(m.get_element_entities()) == 0
    assert len(m.get_node_entities()) == 0



from .mesh import Mesh
from .main_parser import MainParser


def parse(filename: str) -> Mesh:
    """Parse Gmsh .msh file and return `Mesh` object."""
    mesh = Mesh()
    mesh.set_name(filename)
    parser = MainParser()
    with open(filename, "r") as io:
        parser.parse(mesh, io)
    return mesh



from typing import List
from .node import Node


class NodeEntity(object):

    """ NodeEntity class holds nodes for one block. """

    def __init__(self):
        self.dimension_ = -1
        self.tag_ = -1
        self.number_of_parametric_coordinates_ = -1
        self.number_of_nodes_ = -1
        self.nodes_ = {}

    def set_dimension(self, dimension: int):
        """Set the dimension of the entity to `dimension`."""
        self.dimension_ = dimension

    def get_dimension(self) -> int:
        """Get the dimension of the entity."""
        return self.dimension_

    def set_tag(self, tag: int):
        """Set the tag of the entity."""
        self.tag_ = tag

    def get_tag(self) -> int:
        """Get the tag of the entity."""
        return self.tag_

    def set_number_of_parametric_coordinates(self, npar: int):
        """Set the number of parametric coordinates of the entity."""
        self.number_of_parametric_coordinates_ = npar

    def get_number_of_parametric_coordinates(self) -> int:
        """Get the number of parametric coordinates of the entity."""
        return self.number_of_parametric_coordinates_

    def set_number_of_nodes(self, number_of_nodes: int):
        """Set the number of nodes of the entity."""
        self.number_of_nodes_ = number_of_nodes

    def get_number_of_nodes(self) -> int:
        """Get the number of nodes of the entity."""
        return self.number_of_nodes_

    def add_node(self, node: Node):
        """Add new node to entity."""
        self.nodes_[node.get_tag()] = node

    def get_node(self, tag: int) -> Node:
        """Get node from entity by its tag."""
        return self.nodes_[tag]

    def get_nodes(self) -> List[Node]:
        """Get all nodes in this entity."""
        return self.nodes_.values()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

test_deps = [
    'pytest',
    'pytest-cov',
    'pytest-pycodestyle',
    'pytest-pep8',
    'pytest-flake8',
]

extras = {
    'test': test_deps,
}

setuptools.setup(
    name="gmshparser",
    version="0.1.0",
    author="Jukka Aho",
    author_email="ahojukka5@gmail.com",
    description="Package is used to parser Gmsh .msh file format",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ahojukka5/gmshparser",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    tests_require=test_deps,
    extras_require=extras,
)



from typing import TextIO
from .abstract_parser import AbstractParser
from .mesh import Mesh
from .element import Element
from .helpers import parse_ints
from .element_entity import ElementEntity


class ElementsParser(AbstractParser):

    """ElementParser is responsible to parse data between tags $Elements and
    $EndElements."""

    @staticmethod
    def get_section_name():
        return "$Elements"

    @staticmethod
    def parse(mesh: Mesh, io: TextIO) -> None:
        line = io.readline()
        if line.startswith("$Elements"):
            line = io.readline()
        meta = list(map(int, line.strip().split(" ")))

        mesh.set_number_of_element_entities(meta[0])
        mesh.set_number_of_elements(meta[1])
        mesh.set_min_element_tag(meta[2])
        mesh.set_max_element_tag(meta[3])
        for i in range(mesh.get_number_of_element_entities()):
            emeta = parse_ints(io)
            entity = ElementEntity()
            entity.set_dimension(emeta[0])
            entity.set_tag(emeta[1])
            entity.set_element_type(emeta[2])
            entity.set_number_of_elements(emeta[3])
            for j in range(entity.get_number_of_elements()):
                element_info = parse_ints(io)
                element_tag = element_info[0]
                element_connectivity = element_info[1:]
                element = Element()
                element.set_tag(element_tag)
                element.set_connectivity(element_connectivity)
                entity.add_element(element)
            mesh.add_element_entity(entity)



from typing import TextIO
from .abstract_parser import AbstractParser
from .mesh import Mesh


class MeshFormatParser(AbstractParser):

    @staticmethod
    def get_section_name():
        return "$MeshFormat"

    @staticmethod
    def parse(mesh: Mesh, io: TextIO) -> None:
        s = io.readline().strip().split(" ")
        mesh.set_version(float(s[0]))
        mesh.set_ascii(int(s[1]) == 0)
        mesh.set_precision(int(s[2]))



from gmshparser import MainParser
from gmshparser.mesh import Mesh
from io import StringIO
from pytest import raises


__content__ = StringIO("""
$MeshFormat
4.1 0 8
$EndMeshFormat
$Unknown
my custom data
$EndUnknown
""")


__bad_content__ = StringIO("""
$MeshFormat
4.1 0 8          MSH4.1, ASCII
$EndMeshFormat
$Nodes
1 6 1 6          1 entity bloc, 6 nodes total, min/max node tags: 1 and 6
2 1 0 6          2D entity (surface) 1, no parametric coordinates, 6 nodes
$EndNodes
""".strip())


def test_mainparser():
    parser = MainParser()
    mesh = Mesh()
    parser.parse(mesh, __content__)
    assert mesh.get_version() == 4.1


def test_parse_io_bad_content():
    parser = MainParser()
    mesh = Mesh()
    with raises(Exception):
        parser.parse(mesh, __bad_content__)



from typing import TextIO
from .mesh import Mesh


class AbstractParser(object):

    """ AbstractParser is a superclass of all other parsers.

    All other parsers must inheric ``AbstractParser`` and implement their own
    static methods ``parse`` and ``get_section_name``.

    The first argument of the ``parse`` is a mutable ``mesh`` object, which
    parser modifies in-place. The second argument is ``io``, where parser reads
    the text file line by line using `readline()`. Parser must stop reading the
    file to the section end mark, e.g. ``$EndNodes`` in the case of parser
    which is responsible to parse nodes, starting from a section start mark
    ``$Nodes``.

    Another must-to-implement static method is ``get_section_name()``, which
    must return the name of the line where this parser should activate. For
    example, if the section name is ``$Nodes``, then ``get_section_name()``
    must return string ``$Nodes``.
    """

    @staticmethod
    def get_section_name():
        raise NotImplementedError("Not implemented.")

    @staticmethod
    def parse(mesh: Mesh, io: TextIO) -> None:
        raise NotImplementedError("Not implemented.")
